@@ -24,7 +24,14 @@
 #include <ndpi_main.h>
 #include <ndpi_typedefs.h>
 
-void ndpi_global_init(void);
+#ifdef _ND_ENABLE_NDPI_DEBUG
+void nd_ndpi_debug_printf(uint32_t protocol, void *ndpi,
+  ndpi_log_level_t level, const char *file,
+  const char *func, unsigned line, const char *format, ...);
+#endif
+
+void nd_ndpi_global_init(void);
+void nd_ndpi_global_shutdown(void);
 
 struct ndpi_detection_module_struct *nd_ndpi_init(void);
 
