@@ -128,6 +128,10 @@
 #define ND_INTERFACES_PATH \
     ND_PERSISTENT_STATEDIR "/" ND_INTERFACES_BASE
 
+#define ND_ADDR_GROUPS_BASE "address-groups.d"
+#define ND_ADDR_GROUPS_PATH \
+    ND_PERSISTENT_STATEDIR "/" ND_ADDR_GROUPS_BASE
+
 #define ND_LICENSE_MANAGER_BASE "libnetify-plm.so"
 #define ND_LICENSE_MANAGER_PATH \
     ND_PLUGIN_LIBDIR "/" ND_LICENSE_MANAGER_BASE
